library dpay_flutter;

import 'dart:convert';
import 'package:eventify/eventify.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_webview_plugin/flutter_webview_plugin.dart' as flutterWebviewPlugin;
import 'package:logger/logger.dart';
import 'package:webview_flutter/webview_flutter.dart' as webview;

import 'CheckoutOptions.dart';
import 'Constants.dart';

var logger = Logger();
BuildContext globalContext;
// EventEmitter instance used for communication
EventEmitter _eventEmitter;

// Event names
const PAYMENT_SUCCESS = 'paymentSuccess';
const PAYMENT_FAIL = 'paymentError';
const PAYMENT_CLOSE = 'paymentClose';

class Durianpay {

  Durianpay() {
    _eventEmitter = new EventEmitter();
  }

  static String status = "";

  checkout (BuildContext context, DCheckoutOptions checkoutOptions) {

    globalContext = context;
    var buffer = new StringBuffer();
    if(checkoutOptions.accessToken != null) {
      buffer.write("&access_key=");
      buffer.write(checkoutOptions.accessToken);
    }
    if(checkoutOptions.environment != null) {
      buffer.write("&environment=");
      buffer.write(checkoutOptions.environment);
    }
    if(checkoutOptions.locale != null) {
      buffer.write("&locale=");
      buffer.write(checkoutOptions.locale);
    }
    if(checkoutOptions.siteName != null) {
      buffer.write("&site_name=");
      buffer.write(checkoutOptions.siteName);
    }
    if(checkoutOptions.orderId != null) {
      buffer.write("&order_id=");
      buffer.write(checkoutOptions.orderId);
    }
    if(checkoutOptions.customerId != null) {
      buffer.write("&customer_id=");
      buffer.write(checkoutOptions.customerId);
    }
    logger.d("url", buffer.toString());
    var url = buffer.toString();

    return Navigator.push(
      context,
      MaterialPageRoute(builder: (context) => DurianpayCheckout(url: url)),
    );
  }
  /// Registers event listeners for payment events
  void on(String event, Function handler) {
    EventCallback callBack = (event, context) {
      handler(event.eventData);
    };
    _eventEmitter.on(event, null, callBack);
  }

  /// Clears all event listeners
  void clear() {
    if(_eventEmitter != null) {
      _eventEmitter.clear();
    }
  }
}

class DurianpayCheckout extends StatelessWidget {
  var url;
  webview.WebViewController _controller;

  final Set<webview.JavascriptChannel> jsChannels = [
    webview.JavascriptChannel(
        name: 'android',
        onMessageReceived: (webview.JavascriptMessage message) {
          Map<String, dynamic> messageJSON = jsonDecode(message.message);
          logger.i("After parsing : ", messageJSON);

          if(messageJSON[TYPE_IDENTIFIER] != CLOSE_POPUP_RESPONSE) {
            logger.i("closed with type", messageJSON[TYPE_IDENTIFIER]);
            if(messageJSON[TYPE_IDENTIFIER] == SUCCESS_RESPONSE) {
              _eventEmitter.emit(PAYMENT_SUCCESS, null, message.message);
            }else if(messageJSON[TYPE_IDENTIFIER] == CLOSE_RESPONSE){
              _eventEmitter.emit(PAYMENT_CLOSE, null, message.message);
            }else if(messageJSON[TYPE_IDENTIFIER] == FAILURE_RESPONSE){
              _eventEmitter.emit(PAYMENT_FAIL, null, message.message);
            }
            Navigator.pop(globalContext);
          }
        }),
  ].toSet();

  DurianpayCheckout({Key key, this.url}): super(key: key);

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
        onWillPop: () => pressBackButton(context),
        child: Scaffold(
            resizeToAvoidBottomInset: true,
            body: SafeArea(
              child : webview.WebView(
                  initialUrl: CHECKOUT_URL + "?" + url,
                  onWebViewCreated: (webview.WebViewController webViewController) {
                    _controller = webViewController;
                  },
                  javascriptMode: webview.JavascriptMode.unrestricted,
                  javascriptChannels: jsChannels,
                  navigationDelegate: (webview.NavigationRequest request) {
                    Navigator.push(
                      context,
                      MaterialPageRoute(builder: (context) => DurianpaySubView(url: request.url)),
                    );
                    return webview.NavigationDecision.prevent;
                  },
                  onPageFinished: (String url) {
                    print('Page finished loading: $url');
                  })
            )
        ));
  }

  Future<bool> pressBackButton(BuildContext context) async {
    if (await _controller.canGoBack()) {
      print("on will go back");
      _controller.goBack();
      return Future.value(false);
    } else {
      Navigator.pop(globalContext);
    }
  }
}

class DurianpaySubView extends StatelessWidget {
  String url;

  DurianpaySubView({Key key, this.url}): super(key: key);

  final Set<flutterWebviewPlugin.JavascriptChannel> jsChannelsSubView = [
    flutterWebviewPlugin.JavascriptChannel(
        name: 'android',
        onMessageReceived: (flutterWebviewPlugin.JavascriptMessage message) {
          Map<String, dynamic> messageJSON = jsonDecode(message.message);
          logger.i("After parsing in subview : ", messageJSON);

            if(messageJSON[TYPE_IDENTIFIER] == SUCCESS_RESPONSE) {
              _eventEmitter.emit(PAYMENT_SUCCESS, null, message.message);
            }else if(messageJSON[TYPE_IDENTIFIER] == CLOSE_RESPONSE){
              _eventEmitter.emit(PAYMENT_CLOSE, null, message.message);
            }else if(messageJSON[TYPE_IDENTIFIER] == FAILURE_RESPONSE){
              _eventEmitter.emit(PAYMENT_FAIL, null, message.message);
            }

            logger.i("closed with type", messageJSON[TYPE_IDENTIFIER]);
            Navigator.pop(globalContext);
        }),
  ].toSet();

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        body: Builder(builder: (BuildContext context) {
          return flutterWebviewPlugin.WebviewScaffold(
              // initialUrl: url,
            url: url,
            javascriptChannels: jsChannelsSubView,
            useWideViewPort: true,
              // javascriptMode: JavascriptMode.unrestricted,
              // onWebViewCreated: (WebViewController webViewController) {
              //   logger.i("created webview with url : ", url);
              // },
              // onPageFinished: (String url) {
              //   print('Page finished loading: $url');
              // },
          );
        })
    );
  }
}

